# -*- coding: utf-8 -*-
import os
from PyQt5 import QtGui, uic
from PyQt5.QtWidgets import QFileDialog
from qgis.gui import (QgsOptionsPageWidget)
from qgis.PyQt.QtWidgets import  QVBoxLayout
from .qgissettingmanager import *

from ..s3.S3Connection import S3Connection 

from ..qgisutils import (
    QgisLogger
)

WIDGET, BASE = uic.loadUiType(
    os.path.join(os.path.dirname(__file__), 'settings.ui')
)

class ConfigOptionsPage(QgsOptionsPageWidget):

    def __init__(self, parent, settings):
        super(ConfigOptionsPage, self).__init__(parent)
        self.settings = settings
        self.config_widget = ConfigDialog(self.settings)
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setMargin(0)
        self.setLayout(layout)
        layout.addWidget(self.config_widget)
        self.setObjectName('onedoorOptions')

    def apply(self):
        self.config_widget.accept_dialog()
        self.settings.emit_updated()

class ConfigDialog(WIDGET, BASE, SettingDialog):
    def __init__(self, settings):
        super(ConfigDialog, self).__init__(None)
        self.setupUi(self)
        SettingDialog.__init__(self, settings)
        self.settings = settings
        self.test_server_error_label.hide()
        self.test_server_success_label.hide()
        self.test_configuration_error_label.hide()
        self.test_configuration_success_label.hide()
        self.logger = QgisLogger('QgisS3.ConfigDialog')
        
        self.btn_test.clicked.connect(self.test_connection)
        self.orgComboBox.currentIndexChanged.connect(self.org_selected)

        self.configComboBox.currentIndexChanged.connect(self.conf_selected)
        
        
        self.host.textEdited.connect(self.onConnectionChange)
        self.token.textEdited.connect(self.onConnectionChange)
        
        self.test_connection()
        
        self.updatingCombos = False
        self.organisation.hide()
        self.configuration.hide()
    
    def test_connection(self):
        self.test_server_error_label.hide()
        self.test_server_success_label.hide()
        self.test_configuration_error_label.hide()
        self.test_configuration_success_label.hide()
        
        if self.host.text().endswith("/"):
            self.test_server_error_label.show()
            self.test_server_error_label.setText("Error:\nNot a valid API Host. Cannot end with '/' (format: http[s]://domain[:port])")
            return

        connection_info = {
          "host": self.host.text(),
          "token": self.token.text(),
          "organisation": "None",
          "configuration": "None"
        }

        self.s3Connection = S3Connection(connection_info)
        self.description = None
        self.updatingCombos = True
        self.orgComboBox.clear()
        self.configComboBox.clear()
        self.updatingCombos = False
        self.s3Connection.ping_server().then(self.onPingServerSuccess, self.onPingServerError)
        
    def onConnectionChange(self):
        self.updatingCombos = True
        self.orgComboBox.clear()
        self.configComboBox.clear()
        self.updatingCombos = False
        self.test_server_success_label.hide()
        self.test_server_error_label.hide()

    def onPingServerSuccess(self, description):
        self.description = description
        if (len(description["organisations"]) == 0 and len(self.token.text()) == 0):
            self.test_server_error_label.show()
            self.test_server_error_label.setText("<html>Mangler du at angive API Key? Log ind på <a href='" + self.host.text() + "'>" + self.host.text() + "</a> og find din API Key under 'Min profil'</html>")
            self.test_server_error_label.setOpenExternalLinks(True)
        else:
            self.test_server_success_label.show()
            self.test_server_success_label.setText("Forbundet til OneDoor Server")

            #org            
            index = 0
            orgSet = False
            self.updatingCombos = True
            for organisation in description["organisations"]:
                self.orgComboBox.insertItem(index, organisation, organisation)
                if organisation == self.organisation.text():
                    orgSet = True
                    self.orgComboBox.setCurrentIndex(index)
                index = index + 1
            if not orgSet:
                self.orgComboBox.setCurrentIndex(0)
            self.updatingCombos = False
            self.org_selected(index)
            #connect to select and call again
            
    def org_selected(self, index):
        if not self.updatingCombos:
            if index > -1:
                organisationId = self.orgComboBox.currentText()
                self.organisation.setText(organisationId)
                if (self.description):
                    resources = self.description['resources']
                    organisations = self.find_child(resources, 'organisations')
                    if organisations:
                        organisation = self.find_child(organisations, organisationId)
                        if organisation:
                            configurations = self.find_child(organisation, 'configurations')
                            if configurations:
                                #conf
                                self.updatingCombos = True
                                self.configComboBox.clear()
                                index = 0
                                confSet = False
                                for configuration in configurations['children']:
                                    text = configuration['displayname']
                                    if 'description' in configuration:
                                        text = text + ' - ' +  configuration['description'][:100]
                                    self.configComboBox.insertItem(index, text, configuration['id'])
                                    if configuration['id'] == self.configuration.text():
                                        confSet = True
                                        self.configComboBox.setCurrentIndex(index)
                                    index = index + 1
                                if not confSet:
                                    self.configComboBox.setCurrentIndex(0)
                                self.configuration.setText(self.configComboBox.currentData())
                                self.updatingCombos = False
                        
    def conf_selected(self, index):
        if not self.updatingCombos:
            if index > -1:
                self.configuration.setText(self.configComboBox.currentData())
        
    def find_child(self, collection, id):
        for child in collection['children']:
            if child['id'] == id:
                return child
        return None            
    
    def onPingServerError(self, error_message):
        self.description = None
        self.test_server_error_label.show()
        self.test_server_error_label.setText("Ping server error: " + error_message)
        
    def test_configuration(self):    
        self.test_configuration_error_label.hide()
        self.test_configuration_success_label.hide()
        connection_info = {
          "host": self.host.text(),
          "token": self.token.text(),
          "organisation": self.organisation.text(),
          "configuration": self.configuration.text()
        }

        self.s3Connection = S3Connection(connection_info)
        self.s3Connection.ping_configuration().then(self.onConnectionSuccess, self.onConnectionError)
    
    def onConnectionSuccess(self, connection_result):
        self.test_configuration_success_label.show()
        self.test_configuration_success_label.setText("Success")
    
    def onConnectionError(self, error_message):
        self.test_configuration_error_label.show()
        self.test_configuration_error_label.setText("Ping configuration error: " + error_message)

