# -*- coding: utf-8 -*-
import uuid
import json
import collections
from qgis.PyQt.QtCore import QObject
from ..qgisutils import (
    QgisLogger
)



from .history import History
from.s3Client import S3Endpoint

class S3Connection(QObject):

    def __init__(self, connection_info, history_persister = None):
        QObject.__init__(self)
        self.host = connection_info["host"]
        self.token = connection_info["token"]
        self.organisation = connection_info["organisation"]
        self.configuration = connection_info["configuration"]
        self.history = None
        
        self.sources = None
        self.metadata = None
        
        self.S3Endpoint = S3Endpoint(connection_info)

        self.history_persister = history_persister
        self.history = History()
        if self.history_persister:
            self.history.load(self.history_persister.read(self.host, self.organisation, self.configuration))

        self.Thenable = collections.namedtuple('Thenable', 'then')
        self.logger = QgisLogger('QgisS3.S3Connection')
        
    def isUnderAuth(self):
        return (len(self.token) > 0)
        
    def ping_server(self):
        s3Request = self.S3Endpoint.ping_server()
        return self.Thenable(then=lambda onSuccess, onError = None: self.get_response(s3Request, onSuccess, onError))

    def ping_configuration(self):
        s3Request = self.S3Endpoint.ping_configuration()
        return self.Thenable(then=lambda onSuccess, onError = None: self.get_response(s3Request, onSuccess, onError))
        
    def connect(self):
        s3Request = self.S3Endpoint.get_description()
        return self.Thenable(then=lambda onSuccess, onError: self.get_load_description(s3Request, onSuccess, onError))

    def get_load_description(self, s3Request, onSuccess, onError):
        #Error func if s3 site doesn't support "description"
        func = lambda errormessage = None: self.S3Endpoint.get_sources().then(lambda sources: self.load_description(sources, onSuccess, onError), onError)
        s3Request.then(lambda description: self.load_description(description, onSuccess, onError), func)
        
    def load_description(self, data, onSuccess, onError):
        try:
            loaded_data = data #self.load(data)
            #Ok what did we get? description or old fashion sources
            if "sources" in loaded_data:
                self.sources = loaded_data["sources"]
                if "metadata" in loaded_data:
                    self.metadata = loaded_data["metadata"]
                else:
                    #This to catch if these three values aren't under metadata (then construct metadata)
                    if "displayname" in loaded_data:
                        self.metadata = {}
                        self.metadata["displayname"] = loaded_data["displayname"]
                        if "description" in loaded_data:
                            self.metadata["description"] = loaded_data["description"]
                        if "homepage" in loaded_data:
                            self.metadata["homepage"] = loaded_data["homepage"]
            else:
                self.sources = loaded_data
            try:
                onSuccess()
            except:
                pass
        except Exception as e:
            self.logger.critical("S3NetworkManager.load_description: Error: " + str(e) + str(loaded_data))
            if (onError):
                try:
                    onError(str(e))
                except:
                    pass

    def query(self, source, type, query):
        s3Request = self.S3Endpoint.query(source, type, query)
        return self.Thenable(then=lambda onSuccess, onError = None: self.get_response(s3Request, onSuccess, onError))

    def get_complete_result(self, result):
        s3Request = self.S3Endpoint.get_complete_result(result)
        return self.Thenable(then=lambda onSuccess, onError = None: self.get_response(s3Request, onSuccess, onError))

    def get_detail(self, detail):
        s3Request = self.S3Endpoint.get_detail(detail)
        return self.Thenable(then=lambda onSuccess, onError = None: self.get_response(s3Request, onSuccess, onError))

    def get_response(self, s3Request, onSuccess, onError):
        s3Request.then( onSuccess, onError)

    def get_sources(self):
        return self.sources

    def get_metadata(self):
        return self.metadata
    
    def get_config_id(self):
        return self.organisation + "/" + self.configuration
    
    def put_history(self, result, contextResult = None):
        self.history.put_result(result, contextResult)
        if self.history_persister:
            self.history_persister.write(self.history.dump(), self.host, self.organisation, self.configuration)
        
    def get_history(self, filter = None): 
        return self.history.get(filter)   

