# -*- coding: utf-8 -*-
import uuid
import json
import collections
from qgis.PyQt.QtCore import QUrl, QUrlQuery, QObject, QByteArray
from qgis.PyQt.QtNetwork import QNetworkRequest, QNetworkAccessManager
from ..qgisutils import (
    QgisLogger
)

from .history import History



class S3Endpoint(QObject):

    def __init__(self, connection_info):
        QObject.__init__(self)
        self.host = connection_info["host"]
        token = connection_info["token"]
        organisation = connection_info["organisation"]
        configuration = connection_info["configuration"]
        
        self.apiRoot = self.host + "/api/v1"
        self.endpoint = self.apiRoot + "/organisations/" + organisation + "/configurations/" + configuration
        self.Thenable = collections.namedtuple('Thenable', 'then')
        self.s3NetworkManager = S3NetworkManager(token)
        self.logger = QgisLogger('QgisS3.S3Endpoint')
        
    def ping_server(self):
        qurl = QUrl(self.apiRoot + "/description")
        return self.s3NetworkManager.get(qurl)

    def ping_configuration(self):
        qurl = QUrl(self.endpoint + "/description")
        return self.s3NetworkManager.get(qurl)
    
    def query(self, source, type, query):
        #http://localhost:3000/api/v1/organisations/septima/configurations/example/sources/WebGis/types/Skole%20(Kbh)?q=
        if (source is None):
            qurl = QUrl(self.endpoint)
        else:
            qurl = QUrl(self.endpoint + "/sources/" + source + "/types/" + type)
        qUrlQuery = QUrlQuery()
        qUrlQuery.addQueryItem('q', query)
        qurl.setQuery(qUrlQuery )
        return self.s3NetworkManager.get(qurl)
    
    def get_description(self):
        qurl = QUrl(self.endpoint + "/description")
        return self.s3NetworkManager.get(qurl)

    def get_sources(self):
        qurl = QUrl(self.endpoint + "/sources")
        return self.s3NetworkManager.get(qurl)

    def get_complete_result(self, result):
        source = result["source"] 
        type = result["type"]
        id = str(result["id"])
        url = self.endpoint + "/sources/" + source + "/types/" + type +"/" + id
        qurl = QUrl(url)
        return self.s3NetworkManager.get(qurl)

    def get_detail(self, detail):
        href = detail["href"]
        qurl = QUrl(self.host + href)
        return self.s3NetworkManager.get(qurl)

class S3NetworkManager(QObject):
    
    def __init__(self, token):
        self.token = token
        self.authHeader = QByteArray()
        if token:
            self.authHeader.append('Bearer ' + token);
        self.networkManager = QNetworkAccessManager()
        self.replies = {}
        self.Thenable = collections.namedtuple('Thenable', 'then')
        self.logger = QgisLogger('QgisS3.S3NetworkManager')
    
    def get(self, qurl):
        id = str(uuid.uuid4())
        request = QNetworkRequest( qurl )
        if self.authHeader:
            request.setRawHeader(b'Authorization', self.authHeader);
        self.replies[id] = self.networkManager.get(request)
        return self.Thenable(then=lambda onSuccess, onError: self.connect(id, onSuccess, onError))

    def post(self, qurl, data):
        id = str(uuid.uuid4())
        request = QNetworkRequest( qurl )
        if self.authHeader:
            request.setRawHeader(QByteArray(b'Authorization'), self.authHeader);
        self.replies[id] = self.networkManager.post(request, data)
        return self.Thenable(then=lambda onSuccess, onError: self.connect(id, onSuccess, onError))

    def connect(self, id, onSuccess, onError):
        networkReply = self.replies[id]
        func = lambda: self.get_data(id, onSuccess, onError)
        networkReply.finished.connect( func )

    def get_data(self, id, onSuccess, onError):
        networkReply = self.replies[id]
        del self.replies[id]
        networkReply.deleteLater()
        if not networkReply.error():
            try:
                response = networkReply.readAll()
                response_string = str(response, 'utf-8')
                result = json.loads(response_string)
                try: 
                    onSuccess( result )
                except Exception as e:
                    pass
            except Exception as e:
                self.logger.critical("S3NetworkManager.get_data: Error handling: " + response_string + "Exception: " + type(e).__name__ + ":" + str(e))
                if (onError):
                    try:
                        onError("S3NetworkManager.get_data: Invalid response from server:" + str(e))
                    except:
                        pass
        else:
            if (onError):
                error = networkReply.error()
                if error < 100:
                    try:
                        onError("NetworkError: " + networkReply.errorString())
                    except:
                        pass
                else:
                    error_response = ""
                    try:
                        error_response = str(networkReply.readAll(), 'utf-8')
                    except:
                        pass
                    try:
                        onError("NetworkError: " + networkReply.errorString() + "\n" + error_response)
                    except:
                        pass
