# -*- coding: utf-8 -*-
from base64 import b64decode
import os
from qgis.PyQt.QtCore import (
    pyqtSignal,
    QObject,
    Qt,
    QSize,
    QVariant
)
from qgis.PyQt.QtWidgets import (
    QFrame,
    QHBoxLayout,
    QVBoxLayout,
    QGridLayout,
    QLabel,
    QAction,
    QMenu,
    QSizePolicy
)
from qgis.PyQt.QtGui import (
    QPixmap,
    QIcon,
    QMovie,
    QCursor
)
from qgis.core import (
    QgsWkbTypes,
    QgsVectorLayer,
    QgsFeature,
    QgsProject,
    QgsField,
    QgsCoordinateTransform
)

from ..qgisutils import (
    get_current_crs,
    QgisLogger
)

from .detailitem_factory import DetailItemFactory
from builtins import str

class DetailView(QObject):

    on_close_clicked = pyqtSignal()
    on_result_clicked = pyqtSignal(dict, dict)
    on_back_clicked = pyqtSignal(dict)
    on_onedoor_button_clicked = pyqtSignal(dict, str)

    def __init__(self, container, qLocaleToUse):
        super(DetailView, self).__init__()

        self.sources = []
        self.qLocaleToUse = qLocaleToUse
        
        # Get references to ui parts
        self.title = container.result_title
        self.result_description = container.result_description
        self.result_image = container.result_image
        self.toolBox = container.toolBox
        
        self.close_button = container.close_button
        self.close_button.clicked.connect(self.on_close_button_clicked)
        
        self.backbutton = container.backbutton
        self.backbutton.clicked.connect(self.on_backbutton_clicked)
        self.container = container
        self.detailItemFactory = DetailItemFactory(self.container, self.on_detail_result_clicked, self.qLocaleToUse)

        # Load icons
        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'details-close.svg')
        self.close_button.setIcon(QIcon(icon_path))

        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'details-back.svg')
        self.backbutton.setIcon(QIcon(icon_path))
        self.backbutton.resize(20, 20) 

        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'result-createlayer.svg')
        self.mActionCreateFeatureLayer_icon = QIcon(icon_path)
        
        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'result-createtypelayer.svg')
        self.mActionCreateFeatureTypeLayer_icon = QIcon(icon_path)
        
        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'spinner.gif')
        self.loading_movie = QMovie(icon_path)

        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'chevron_down.svg')
        self.chevron_down = QIcon(icon_path)

        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'chevron_right.svg')
        self.chevron_right = QIcon(icon_path)
        
        self.favorite_button = container.favorite_button
        favorite_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'star-full.svg')
        self.favorite_icon = QIcon(favorite_icon_path)

        not_favorite_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'star-outline.svg')
        self.not_favorite_icon = QIcon(not_favorite_icon_path)
        self.favorite_button.setIcon(self.not_favorite_icon)
        self.favorite_button.setToolTip(self.tr("Save as favorite"))
        
        self.onedoor_button = container.onedoor_button
        onedoor_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'open-in-onedoor.svg')
        self.onedoor_icon = QIcon(onedoor_icon_path)
        self.onedoor_button.setIcon(self.onedoor_icon)
        self.onedoor_button.setToolTip(self.tr("See in OneDoor Web"))
        self.onedoor_button.clicked.connect(
            self.onedoor_button_clicked
        )
        
        #TBD: https://github.com/Septima/qgis-s3/issues/56#issuecomment-870347275
        #self.onedoor_button.hide()
        
        self.pdf_button = container.pdf_button
        pdf_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'open-in-pdf.svg')
        self.pdf_icon = QIcon(pdf_icon_path)
        self.pdf_button.setIcon(self.pdf_icon)
        self.pdf_button.setToolTip(self.tr("Download as PDF"))
        
        
        # Set up Save to layer button
        #https://stackoverflow.com/questions/5695313/qt-drop-down-button
        self.btnSaveToLayer = container.btnSaveScratchLayer
        self.btnSaveToLayer.setIcon(self.mActionCreateFeatureLayer_icon)
        self.btnSaveToLayer.setToolTip( self.tr("Save to layer" ))
        menu = QMenu();
        self.feature_scratch_action = QAction(self.mActionCreateFeatureLayer_icon, "", self.container.qgisIface.mainWindow());
        self.type_scratch_action = QAction(self.mActionCreateFeatureTypeLayer_icon, "", self.container.qgisIface.mainWindow());
        menu.addAction(self.feature_scratch_action);
        menu.addAction(self.type_scratch_action);
        self.btnSaveToLayer.setMenu(menu);

        self.btnSaveToLayer.clicked.connect(
            self.actionCreateFeatureLayer
        )
        
        self.feature_scratch_action.triggered.connect(
            self.actionCreateFeatureLayer
        )
        self.type_scratch_action.triggered.connect(
            self.actionCreateFeatureTypeLayer
        )

        #self.setS3Connection(s3Connection)
        #self.s3Connection = s3Connection
        self.toolBox.currentChanged.connect(self.show_tab)
        self.tabs = {}
        self.reset()
        
        self.current_result = None
        
        self.history = []
        self.last_detail_id = None
        
        self.defaultSRID = None
        
    def setDefaultSRID(self, srid):
        self.defaultSRID = srid
        
    def setS3Connection(self, s3Connection):
        self.s3Connection = s3Connection
        #https://github.com/Septima/qgis-s3/issues/56
        #if (self.s3Connection.isUnderAuth()):
        #    self.onedoor_button.hide()
        #else:
        #    self.onedoor_button.show()
        
    def showAdvanced(self, boolShow):
        if boolShow:
            self.favorite_button.show()
            self.pdf_button.show()
        else:
            self.favorite_button.hide()
            self.pdf_button.hide()

    def actionCreateFeatureLayer(self):
        self.btnSaveToLayer.clicked.disconnect()
        self.btnSaveToLayer.setIcon(self.mActionCreateFeatureLayer_icon)
        self.btnSaveToLayer.clicked.connect(
            self.actionCreateFeatureLayer
        )
        self.saveToLayer(False)

    def actionCreateFeatureTypeLayer(self):
        self.btnSaveToLayer.clicked.disconnect()
        self.btnSaveToLayer.setIcon(self.mActionCreateFeatureTypeLayer_icon)
        self.btnSaveToLayer.clicked.connect(
            self.actionCreateFeatureTypeLayer
        )
        self.saveToLayer(True)

    def fixSaveToLayerButtons(self):
        if self.current_result['qGisGeom']:
            title = str(self.current_result['title'])
            self.feature_scratch_action.setText(self.tr("Save to layer") + ": '" + title +"'")
            type = self.current_result['type']
            self.type_scratch_action.setText(self.tr("Save to layer") + ": '" + type +"'")
            self.btnSaveToLayer.show()
        else:
            self.btnSaveToLayer.hide()

    def reset(self):
        initital_tab_count = self.toolBox.count()
        for index in range(initital_tab_count):
            self.toolBox.removeItem(index)
        self.tabs = {}

    def set_sources(self, sources):
        self.sources = sources
        
    def saveToLayer(self, in_type_layer = True):
        #https://gis.stackexchange.com/questions/60473/how-can-i-programatically-create-and-add-features-to-a-memory-layer-in-qgis-1-9
        if in_type_layer:
            vl = self.get_type_layer()
        else:
            vl = self.get_feature_layer()

        pr = vl.dataProvider()

        vl.startEditing()

        geom = self.current_result['qGisGeom']
        
        # add a feature
        fet = QgsFeature()
        fet.setGeometry(geom)
        title = self.current_result['title']
        fet.setAttributes([title])
        pr.addFeatures([fet])
        vl.commitChanges()
        vl.updateExtents()
        vl.removeSelection
        vl.select(fet.id())

        #Label: https://gis.stackexchange.com/questions/136526/how-to-add-label-to-qgsvectorlayer-by-python

    def get_type_layer(self):
        title = self.current_result['type']
        layers = QgsProject.instance().mapLayersByName(title)
        if layers:
            return layers[0]
        #Compute url
        type = QgsWkbTypes.displayString(self.current_result['qGisGeom'].wkbType())
        dstCrs = get_current_crs(self.container.qgisIface)
        authid = dstCrs.authid()
        url = type +'?crs=' + authid
        vl = QgsVectorLayer(url, title, "memory")
        
        vl.setCustomProperty("skipMemoryLayersCheck", True)
        pr = vl.dataProvider()
        pr.addAttributes([QgsField(self.tr("title"), QVariant.String)])
        QgsProject.instance().addMapLayer(vl)
        return vl

    def get_feature_layer(self):
        title = self.current_result['title']
        #Compute url
        type = QgsWkbTypes.displayString(self.current_result['qGisGeom'].wkbType())
        dstCrs = get_current_crs(self.container.qgisIface)
        authid = dstCrs.authid()
        url = type +'?crs=' + authid
        vl = QgsVectorLayer(url, title, "memory")

        vl.setCustomProperty("skipMemoryLayersCheck", True)
        pr = vl.dataProvider()
        pr.addAttributes([QgsField(self.tr("title"), QVariant.String)])
        QgsProject.instance().addMapLayer(vl)
        return vl


    def on_close_button_clicked(self):
        self.on_close_clicked.emit()

    def on_backbutton_clicked(self):
        if len(self.history) > 0:
            new_result = self.history.pop()
            self.on_back_clicked.emit(new_result)
            self.show_result(new_result)
        else:
            self.on_close_clicked.emit()

    def on_detail_result_clicked(self, result):
        self.on_result_clicked.emit(result, self.current_result)
        
    def onedoor_button_clicked(self):
        self.on_onedoor_button_clicked.emit(self.current_result, self.last_detail_id)

    def set_result(self, result):
        if self.current_result:
            self.history.append(self.current_result)
        
        self.show_result(result)

    def show_result(self, result):
        self.last_detail_id = None
        if len(self.history) > 0:
            prev_result_title = self.history[len(self.history)-1]["title"]
            self.backbutton.setToolTip(self.tr("Return to") + " " + prev_result_title)
        else:
            self.backbutton.setToolTip(self.tr("Close"))

        for index in range(len(self.tabs)):
            self.tabs[index]["tabContent"].setParent(None)
        self.reset()
        self.current_result = result

        self.fixSaveToLayerButtons()

        image_data = b64decode(result['image'].split(',')[1])
        pixmap = QPixmap()
        succes_loading_image = pixmap.loadFromData(image_data)
        if succes_loading_image:
            pixmap = pixmap.scaled(QSize(20, 20),  Qt.KeepAspectRatio)
            self.result_image.setPixmap(pixmap)
        self.title.setText(str(self.current_result["title"]))
        #if "description" in self.current_result and self.current_result["description"] != "":
        if self.current_result["description"]:
            self.result_description.setText(self.current_result["description"] + " - " + self.get_type_displayname(self.current_result["source"], self.current_result["type"]))
        else:
            self.result_description.setText(self.get_type_displayname(self.current_result["source"], self.current_result["type"]))

        if "details" in result:
            detail_count = len(result["details"])
            for index in range(len(result["details"])):
                detail = result["details"][index]
                
                tabContent = QFrame(self.container)
                grid_layout = QGridLayout()
                grid_layout.setContentsMargins(4, 4, 0, 0)
                grid_layout.setColumnStretch(1, 1)
                tabContent.setLayout(grid_layout)
                #Stylesheet
                tabContent.setProperty("class", "detailTabContent")
                #grid_layout.setRowStretch(2, 1)
                
                loading_label = QLabel("", self.container)
                #loading_label.setPixmap(self.loading_pixmap)
                loading_label.setMovie(self.loading_movie)

                grid_layout.addWidget(loading_label, 0, 0, Qt.AlignTop)
                self.loading_movie.start()

                empty = QLabel()
                empty.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding)
                grid_layout.addWidget(empty, 1, 1);
                
                image_data = b64decode(detail['image'].split(',')[1])
                pixmap = QPixmap()
                succes_loading_image = pixmap.loadFromData(image_data)
                if succes_loading_image:
                    pixmap = pixmap.scaled(QSize(25, 25),  Qt.KeepAspectRatio)
                    #icon = QIcon(pixmap)
                    #empty_icon = QIcon("bla")
                    tabIndex = self.toolBox.addItem(
                        tabContent, self.chevron_down, detail["title"])
                        #tabContent, icon, detail["title"])
                else:
                    tabIndex = self.toolBox.addItem(
                        tabContent, detail["title"])
                self.tabs[tabIndex] = {
                    "tabContent": tabContent, "isRendered": False, "detail": detail}
            self.show_tab(0)

    def get_type_displayname(self, source, type):
        for known_source in self.sources:
            if known_source["source"] == source:
                for known_type in known_source["types"]:
                    if known_type["id"] == type:
                        return known_type["singular"]
        return type
    
    def show_tab(self, tab_index):
        if tab_index > -1:
            if tab_index in self.tabs:
                detail = self.tabs[tab_index]["detail"]
                self.last_detail_id = detail["id"]
                if not self.tabs[tab_index]["isRendered"]:
                    if detail["href"]:
                        self.s3Connection.get_detail(detail).then(
                            lambda items: self.render_items_to_tab(tab_index, items),
                            lambda error: self.render_error_to_tab(tab_index, error))
                    else:
                        self.render_items(tab_index, detail)
        for i in range(self.toolBox.count()):
            if tab_index == i:
                self.toolBox.setItemIcon(i, self.chevron_right)
            else:
                self.toolBox.setItemIcon(i, self.chevron_down)

    def render_error_to_tab(self, tab_index, error):
        tab_container = self.tabs[tab_index]["tabContent"]
        grid_layout = tab_container.layout() 
        for i in reversed(range(grid_layout.count())): 
            grid_layout.itemAt(i).widget().setParent(None)
        #grid_layout = QGridLayout()
        grid_layout.setColumnStretch(2, 1)
            
        grid_layout.addWidget(
            QLabel(str(error), self.container), 0, 1, 1, 2)
        #tab_container.setLayout(grid_layout)
        grid_layout.setRowStretch(1, 1)
        
    def render_items_to_tab(self, tab_index, items):
        try:
            #https://stackoverflow.com/questions/5395266/removing-widgets-from-qgridlayout
            item_count = len(items)
            if item_count > 0:
                tab_container = self.tabs[tab_index]["tabContent"]
                grid_layout = tab_container.layout() 
                for i in reversed(range(grid_layout.count())):
                    if grid_layout.itemAt(i).widget(): 
                        grid_layout.itemAt(i).widget().setParent(None)

                current_row = 0
                for index in range(item_count):
                    item = items[index]
                    rendered_rows = self.detailItemFactory.render_item(item, grid_layout, current_row, 0)
                    current_row = current_row + rendered_rows
                    
                grid_layout.setRowStretch(current_row, 1)

            self.tabs[tab_index]["isRendered"] = True
        except Exception as e:
            raise Exception ("Exception in DetailView.render_items_to_tab: " + str(e))



