# -*- coding: utf-8 -*-
from base64 import b64decode

from qgis.PyQt.QtCore import (
    pyqtSignal,
    QObject,
    QTimer,
    QPoint,
    QEvent,
    Qt
)
from qgis.PyQt.QtWidgets import (
    QLabel,
    QTreeWidgetItem,
    QApplication,
    QPushButton,
    QFrame,
    QTreeWidget
)
from qgis.PyQt.QtGui import (
    QPalette,
    QPixmap,
    QIcon,
    QKeyEvent
)

class ResultList(QObject):
    
    on_select = pyqtSignal(object)
    on_settings_selected = pyqtSignal()
    
    def __init__(self, container):
        super(ResultList, self).__init__()
        self.treeWidget = container.resultList
        self.errorMessage = container.errorMessage
        #self.treeWidget = QTreeWidget(parent)
        self.treeWidget.setColumnCount(1)
        self.treeWidget.setUniformRowHeights(True)
        self.treeWidget.setRootIsDecorated(False)
        self.treeWidget.setEditTriggers(QTreeWidget.NoEditTriggers)
        self.treeWidget.setSelectionBehavior(QTreeWidget.SelectRows)
        self.treeWidget.setFrameStyle(QFrame.Box | QFrame.Plain)
        self.treeWidget.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        
        self.treeWidget.header().hide()
        self.treeWidget.installEventFilter(self)
        self.treeWidget.setMouseTracking(True)
        
        #self.treeWidget.setWindowFlags(Qt.Popup)
        self.treeWidget.setFocusPolicy(Qt.NoFocus)
        #self.treeWidget.setFocusProxy(parent)
        
        self.treeWidget.itemClicked.connect(self.on_treeWidget_select)
        
        self.errorMessage.hide()

    def eventFilter(self, obj, ev):
        consumed = False
        if obj != self.treeWidget:
            return consumed

        if ev.type() == QEvent.KeyPress:
            key = ev.key()
            if key == Qt.Key_Enter or key == Qt.Key_Return:
                self.on_treeWidget_select()
                consumed = True
            return consumed

        return False

    def on_treeWidget_select(self):
        item = self.treeWidget.currentItem()
        selectedObject = item.data(2, Qt.UserRole)
        self.on_select.emit(selectedObject)
        
    def errorLink(self, link):
        if link == "options":
            self.on_settings_selected.emit()

    def show_error(self, errorText):
        self.errorMessage.setTextFormat(Qt.RichText)
        self.errorMessage.setText(errorText)
        self.errorMessage.show()

        self.errorMessage.linkActivated.connect(self.errorLink)

        self.treeWidget.setUpdatesEnabled(False)
        self.treeWidget.clear()

        #item = QTreeWidgetItem(self.treeWidget)
        #item.setText(0, obj)
        #item.setTextAlignment(0, Qt.AlignLeft)
        
        self.treeWidget.setUpdatesEnabled(True)
        
    def show_results(self, results):

        self.errorMessage.hide()
        
        self.results = results
        
        if results is None or len(results) < 1:
            return

        self.treeWidget.setUpdatesEnabled(False)
        self.treeWidget.clear()

        for result in results:
            item = QTreeWidgetItem(self.treeWidget)

            try:
                image_data = b64decode(
                    result['image'].split(',')[1]
                )
                pixmap = QPixmap()
                succes_loading_image = pixmap.loadFromData(image_data)
                if succes_loading_image:
                   item_icon = QIcon(pixmap)
                   item.setIcon(0, item_icon)
            # Can't decode the base64 image data
            except TypeError as e:
                self.logger.warning('Error decoding base64 icon data.')
                self.logger.warning('Error: {}'.format(e))
            # The key (image) doesnt exist
            except KeyError as e:
                self.logger.warning('Key error in dict.')
                self.logger.warning('Error: {}'.format(e))

            text_to_display = result['title']
            if result['description'] and result['description'] != "":
                item.setToolTip(0, result['description'])
                text_to_display = text_to_display + " - " + result['description']
            item.setText(0, text_to_display)

            item.setTextAlignment(1, Qt.AlignRight)
            #item.setTextColor(1, color)

            item.setData(2, Qt.UserRole, result)

        self.treeWidget.setCurrentItem(self.treeWidget.topLevelItem(0))
        self.treeWidget.resizeColumnToContents(0)
        self.treeWidget.setUpdatesEnabled(True)

    def show(self):
        self.treeWidget.show()

    def setWidth(self, minWidth):
        h = self.treeWidget.sizeHintForRow(0) * min(15, len(self.results)) + 3
        w = max(self.treeWidget.width(), minWidth)
        self.treeWidget.resize(w, h)
                
    def move(self, pos):
        self.treeWidget.move(pos)
        
    def setFocus(self):
        self.treeWidget.setFocus()
        
    def hide(self):
        self.treeWidget.hide()
        
    def clear(self):
        self.treeWidget.clear()
        self.results = None
        
    def unload(self):
        self.treeWidget.removeEventFilter(self)


