 # -*- coding: utf-8 -*-
from base64 import b64decode
from urllib.request import urlopen
from qgis.PyQt.QtCore import (
    Qt,
    QDateTime,
    QSize
)

from qgis.PyQt.QtWidgets import (
    QLabel,
    QPushButton,
    QWidget,
    QHBoxLayout,
    QSpacerItem,
    QSizePolicy
)

from qgis.PyQt.QtGui import (
    QPixmap,
    QIcon,
    QCursor
)

class WidgetFactory():
    def __init__(self, container, qLocaleToUse):
        self.container = container
        self.qLocaleToUse = qLocaleToUse
        
    def createInfoItemWidget(self, infoItem):
        if infoItem["type"] == "labelvalue":
            return self.create_value_widget(infoItem)
        if infoItem["type"] == "result":
            return self.create_result_widget(infoItem["result"])
        if infoItem["type"] == "link":
            return self.create_link_widget(infoItem)
        return QLabel()

    def create_value_widget(self, item):
        try:
            if "value" in item and item["value"]:
                label_text = str(item["value"])
                try:
                    if "valueformat" in item and item["valueformat"] in ["int", "currencyamount", "geometry", "iso-date", "iso-datetime"]:
                        if item["valueformat"] == "int":
                            if "valueunit" in item:
                                if item["valueunit"] in ["m"]:
                                    intVal = int(item["value"])
                                    if (intVal > 999):
                                        doubleVal = intVal / 1000
                                        label_text = self.qLocaleToUse.toString(doubleVal, "f", 1) + " km"
                                    else:
                                        label_text = self.qLocaleToUse.toString(intVal) + " m"
                                else:
                                    label_text = self.qLocaleToUse.toString(int(item["value"])) + " " + item["valueunit"]
                            else:
                                label_text = self.qLocaleToUse.toString(int(item["value"]))
                        if item["valueformat"] == "currencyamount":
                            label_text = self.qLocaleToUse.toCurrencyString(float(item["value"]), ' ')
                        if item["valueformat"] == "iso-date":
                            date = QDateTime.fromString(str(item["value"])[0:10], "yyyy-MM-dd")
                            label_text = self.qLocaleToUse.toString(date, "d/M/yyyy")
                        if item["valueformat"] == "iso-datetime":
                            date = QDateTime.fromString(str(item["value"])[0:19], "yyyy-MM-ddThh:mm:ss")
                            label_text = self.qLocaleToUse.toString(date, "d/M/yyyy h:mm")
                        if item["valueformat"] == "geometry":
                            label_text = ""
                except Exception as e:
                    pass

                label = QLabel(label_text, self.container)
                label.setTextInteractionFlags(Qt.TextSelectableByMouse)
                label.setWordWrap(True)
                if "valueformat" in item:
                    if item["valueformat"] in ["int", "currencyamount", "iso-date", "iso-datetime"]:
                        label.setAlignment(Qt.AlignRight)
                return label
            return QLabel("", self.container)

        except Exception as e:
            raise Exception ("Exception in WidgetFactory.create_value_widget: " + str(e))

    def create_link_widget(self, item):

        try:
            widget = QWidget()
            layout = QHBoxLayout()
            layout.setContentsMargins(0, 0, 0, 0)
            layout.setAlignment(Qt.AlignLeft)
            widget.setLayout(layout)

            spaceritem = QSpacerItem(1,1, QSizePolicy.Expanding, QSizePolicy.Fixed)
            
            if "icon" in item:
                icon = self.create_icon_label(item["icon"])
                if icon:
                    layout.insertWidget(0, icon)
                    
            label = QLabel()
            label.setOpenExternalLinks(True)
            url = item["link"]
            title = str(item["linkTitle"])
            label.setText('<a href="' + url + '">' + title + '</a>')
            layout.insertWidget(1, label)
            
            return widget

        except Exception as e:
            raise Exception ("Exception in WidgetFactory.create_link_widget: " + str(e))

    def create_result_widget(self, result):
        
        result_pixmap = None
        if "image" in result:
            image_data = b64decode(result['image'].split(',')[1])
            pixmap = QPixmap()
            succes_loading_image = pixmap.loadFromData(image_data)
            if succes_loading_image:
                result_pixmap = pixmap.scaled(QSize(20, 20),  Qt.KeepAspectRatio)
                
        if (len(str(result["title"])) > 35):
            buttonText = str(result["title"])[:35] + "..."
        else:
            buttonText = str(result["title"])
        
        if result_pixmap:
            resultWidget = QPushButton(QIcon(result_pixmap), buttonText, self.container)
        else:
            resultWidget = QPushButton(buttonText, self.container)

        resultWidget.setProperty("class", "resultButton")
            
        resultWidget.setToolTip("Gå til " + str(result["title"]))
            
        resultWidget.setCursor(QCursor(Qt.PointingHandCursor))
        
        return resultWidget
    
    def create_icon_label(self, icon_def):
        try:
            image_data = b64decode(icon_def.split(',')[1])
            pixmap = QPixmap()
            succes_loading_image = pixmap.loadFromData(image_data)
            if succes_loading_image:
                pixmap = pixmap.scaled(QSize(20, 20),  Qt.KeepAspectRatio)
                label = QLabel()
                label.setPixmap(pixmap)
                return label
            else:
                return None
        except Exception as e:
            raise Exception ("Exception in WidgetFactory.create_icon_label: " + str(e))

    def create_image_label(self, url):
        #image_data = b64decode(icon_def.split(',')[1])
        #image_data = b64decode(icon_def.split(',')[1])
        #Better:
        # https://stackoverflow.com/questions/8211982/qt-resizing-a-qlabel-containing-a-qpixmap-while-keeping-its-aspect-ratio
        # https://stackoverflow.com/questions/962640/pyqt-qlabel-inheriting
        # https://stackoverflow.com/questions/21041941/how-to-autoresize-qlabel-pixmap-keeping-ratio-without-using-classes
        try:
            data = urlopen(url).read()
            pixmap = QPixmap()
            succes_loading_image = pixmap.loadFromData(data)
            if succes_loading_image:
                pixmap = pixmap.scaledToHeight(300)
                label = QLabel()
                label.setPixmap(pixmap)
                #label.setScaledContents(True)
                #label.setSizePolicy( QSizePolicy.Ignored, QSizePolicy.Ignored )
                #label.resized.connect(self.donothing)
                return label
            else:
                return None
        except Exception as e:
            raise Exception ("Exception in WidgetFactory.create_image_label: " + str(e))

    def donothing(self):
        pass
    
    def create_label(self, text):
        try:
            if (text and text.strip()) != "":
                label = QLabel(text, self.container) 
                label.setAlignment(Qt.AlignTop)
                return label
            else:
                return QLabel("notext_label", self.container)
        except Exception as e:
            raise Exception ("Exception in WidgetFactory.creat_label: " + str(e))